﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using UnityEngine.SceneManagement;  // シーンの切り替えに使用
using UnityEngine.UI;  // UIを使用するのに使用

//////////////////////////////////////////////////////////////////
//【クリアシーンで使用する機能に関するクラス】
// 経過時間とベストタイムの管理・表示や、リスタート処理実行
//////////////////////////////////////////////////////////////////
public class ClearManager : MonoBehaviour
{
    /////////////////////////////////
    // 【変数の設定】
    /////////////////////////////////

    //【Unity Editorから設定するPublic変数】
    public Text text_Time;  // UIに経過時間を表示するテキスト
    public Text text_BestTime;  // UIにベストタイムを表示するテキスト


    /////////////////////////////////
    // 【関数の設定】
    /////////////////////////////////

    //【ゲーム開始時に１回だけ動く関数】
    //（Unityの標準関数）
    void Start()
    {
        // GameMainシーンでの経過時間を取得
        float time = GameManager.OnGet_Time();
        // 保存されたベストタイムを取得
        float time_BestTime = 
            PlayerPrefs.GetFloat("BEST_TIME", 0);
        
        // ベストタイムが保存されてない場合、今回の経過時間を使用
        if (time_BestTime == 0) time_BestTime = time;

        // 今回の経過時間がベストタイムより短ければ、
        // 今回のタイムをベストタイムとして保存
        if (time <= time_BestTime)
        {
            time_BestTime = time;
            PlayerPrefs.SetFloat("BEST_TIME", time);
        }
            
        // 画面上に経過時間とベストタイムを表示
        text_Time.text = time.ToString("###.##");
        text_BestTime.text = time_BestTime.ToString("###.##");
    }


    //【リスタート用ボタンが押された際の処理関数】
    public void OnClickRestart()
    {
        // GameMainシーンを呼び出す
        SceneManager.LoadScene("GameMain");
    }


    //【ベストタイムを初期化する関数】
    public void OnClickReset()
    {
        // ベストタイムとして初期値を保存
        PlayerPrefs.SetFloat("BEST_TIME", 0);
    }
}
