﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

using UnityEngine.SceneManagement;  // シーンの切り替えに使用

//////////////////////////////////////////////////////////////////
//【Playerのコントロールに使用するクラス】
// Playerをキーボード処理に合わせ移動させ、Playerの衝突や通過処理を実行
//////////////////////////////////////////////////////////////////
public class PlayerController : MonoBehaviour
{
    /////////////////////////////////
    // 【変数の設定】
    /////////////////////////////////

    //【定数の設定】（レベル設計で変更可能）
    const float WALK_FORCE = 9;  // 横に進む時にかける力の強さ
    const float JUMP_FORCE = 9;  // ジャンプ時にかける力の強さ
    const float MAX_WALK_SPEED = 1.3f;  // 横に進む速度の最大値
    //（横方向に力をかけ続けると速くなり過ぎるため最大値を設定。）
    const float THRESHOLD = 0.2f;  // スマホ用加速度センサがこの数値までは水平とする

    //【Unity Editorから設定するPublic変数】
    public GameManager class_GameManager;  // GameManagerの関数呼び出し用
    public Rigidbody2D rigidBody;  // Rigidbody2Dコンポーネントの変数や関数の呼び出し用
    public Animator animator;  // Animatorコンポーネントの変数や関数の呼び出し用


    /////////////////////////////////
    // 【関数の設定】
    /////////////////////////////////

    //【毎フレームに１度動く関数】
    //（Unityの標準関数）
    void Update()
    {
        // スペースキーかマウスをクリック又はスマホ画面をタップした場合
        if (Input.GetKeyDown(KeyCode.Space) || Input.GetMouseButtonDown(0))
        {
            // 上方向の速度がゼロの場合
            if (rigidBody.velocity.y == 0)
            {
                // Playerに上向きの力を瞬間的にかける
                rigidBody.AddForce(transform.up * JUMP_FORCE, ForceMode2D.Impulse);
                // アニメーションのパラメータonGroundにfalseをセット
                animator.SetBool("onGround", false);
            } 
        }
    }


    //【固定時間単位に１度動く関数】
    //（Unityの標準関数）
    void FixedUpdate()
    {
        int direction = 0; // Playerの横向きの方向を示すdirection変数、初期値をゼロ
        // （右方向を向いている場合１、左方向を向いている場合ー１）
        
        // 右矢印が押された場合、direction変数を１に
        if (Input.GetKey(KeyCode.RightArrow)) direction = 1;
        // 左矢印が押された場合、direction変数をー１に
        if (Input.GetKey(KeyCode.LeftArrow)) direction = -1;
        // スマホが右方向に一定角度以上傾けられた場合、direction変数を１に
        if (Input.acceleration.x > THRESHOLD) direction = 1;
        // スマホが左方向に一定角度以上傾けられた場合、direction変数をー１に
        if (Input.acceleration.x < -THRESHOLD) direction = -1;

        // speedx変数にPlayerの横方向のスピード（絶対値）を代入
        float speedx = Mathf.Abs(rigidBody.velocity.x);

        // 横方向のスピードがMAX_WALK_SPEEDのスピードより小さい場合
        if (speedx < MAX_WALK_SPEED)
        {
            // Playerにdirection変数が１の場合は右に、-１の場合は左に力をかける
            rigidBody.AddForce(transform.right * direction * WALK_FORCE);
        }

        // direction変数がゼロ以外の場合
        if (direction != 0)
        {
            // direction変数が１の場合は右に、-１の場合は左にPlayerのSpriteの向きを変える
            transform.localScale = new Vector3(0.1f * direction, 0.1f, 1);
        }
    }


    //【Playerのコライダーが他のコライダーに衝突した場合呼ばれる関数】
    //（Unityの標準関数）
    void OnCollisionEnter2D(Collision2D other)
    {
        // 衝突した相手のゲームオブジェクトのタグがBlock（床）の場合
        if (other.gameObject.tag == "Block")
        {
            // アニメーションのパラメータonGroundにtrueをセット
            animator.SetBool("onGround", true);
        }

        // 衝突した相手のゲームオブジェクトのタグがBorder（落下時の境界）の場合
        if (other.gameObject.tag == "Border")
        {
            // GameManagerのGameOver関数を呼び出す
            class_GameManager.GameOver();
        }

        // 衝突した相手のゲームオブジェクトのタグがEnemy_Spider（クモ）の場合
        if (other.gameObject.tag == "Enemy_Spider")
        {
            // 衝突した相手のゲームオブジェクト（クモ）の中心より少し低い位置で衝突した場合
            if (transform.position.y < other.gameObject.transform.position.y - 0.5f)
            {
                // 衝突した相手を消す
                Destroy(other.gameObject);
            }
            // それ以外の場合
            else
            {
                // GameManagerのGameOver関数を呼び出す
                class_GameManager.GameOver();
            }
        }

        // 衝突した相手のゲームオブジェクトのタグがEnemyの場合
        if (other.gameObject.tag == "Enemy")
        {
            // 衝突した相手のゲームオブジェクト(ヘビ)の中心より高い位置で衝突した場合
            if (transform.position.y > other.gameObject.transform.position.y + 0.5f)
            {
                // Playerに上向きの瞬間的な力をかける
                rigidBody.AddForce(Vector2.up * JUMP_FORCE, ForceMode2D.Impulse);
                // 衝突した相手を消す
                Destroy(other.gameObject);
            }
            // それ以外の場合
            else
            {
                // GameManagerのGameOver関数を呼び出す
                class_GameManager.GameOver();
            }
        }
    }


    //【Playerのコライダーが他のコライダーを通過した場合呼ばれる関数】
    //（Unityの標準関数）
    void OnTriggerEnter2D(Collider2D other)
    {
        // 通過した相手のゲームオブジェクトのタグがDoor（出口）の場合
        if (other.gameObject.tag == "Door")
        {
            // GameClearシーンを呼び出す
            SceneManager.LoadScene("GameClear");
        }
    }
}
